/*
 *  libdream robotfindskitten for SEGA Dreamcast
 *  drw <douglas@min.net> 2001
 */

#include <dream.h>
#include "messages.h"
#include "fontset.h"

#define STUFF 15
#define KITTEN "\n\nYOU FOUND KITTEN! WAY TO GO, ROBOT!"

#define INSTRUCTIONS "robotfindskitten
\x7f 1997, 2000 Leonard Richardson\n
SEGA DREAMCAST ADAPTATION
\x7f 2001 DRWIII <DOUGLAS@MIN.NET>\n
Written originally for the Nerth
Pork robotfindskitten contest. In
this game, you are robot (#). Your
job is to find kitten. This task is
complicated by the existance of
various things which are not kitten.
Robot must touch items to determine
if they are kitten or not. The game
ends when robotfindskitten.
Alternatively, you may end the game
by hitting the start button. See the
documentation for more information.\n
        PRESS START BUTTON!"

unsigned long seed=0;
uint8 mcont;
cont_cond_t cond;
struct stat
{
	int x, y, chr, Msg, IsKitten;
	unsigned short color;
} objs[STUFF];

int randnum(int limit)
{
	seed = seed * 1164525 + 1013904223;	// EVIL. anyone know how or where
	return (seed & 0x7fffffff) % limit;	// to poll the RTC in this thing?
}

unsigned short GetColor(int r, int g, int b)
{
	return ( ((r >> 3) << 11) | ((g >> 2) << 5) | ((b >> 3) << 0) );
}

void DrawLine(int y, int color)
{
	int z;

	for (z=0; z<320; z++)
	{
		vram_s[y*320+z] = color;
	}
}

void DrawChar(int x, int y, int offset, unsigned short color)
{
	int dx, dy;

	for (dx=0; dx<8; dx++)
	{
		for (dy=0; dy<8; dy++)
		{
			if (fontset[dy*8+dx+(64*offset)] == '.')
				vram_s[(dy+y)*320+(dx+x)] = color;
		}
	}
}

void DrawString(int x, int y, char *string, unsigned short color)
{
	int act, cur, cr = x;

	while (*string)
	{
		cur = *string;

		if (cur >= 91 && cur <= 96)
			cur = cur;
		else if (cur >= 97 && cur <= 122)
			cur -= 32;
		else if (cur >= 123)
			cur -= 26;

		act = cur - 32;

		if (cur == '\n')
		{
			x = cr;
			y += 10;
		}
		else
		{
			DrawChar(x,y,act,color);
			x += 8;
		}

		string++;
	}
}

void ShadeString(int x, int y, char *string, int white)
{
	DrawString(x+2,y+2,string,GetColor(0,0,0));
	DrawString(x+1,y+1,string,GetColor(white/4,white/4,63));
	DrawString(x,y,string,GetColor(white,white,255));
}

void FadeString(int x, int y, char *string, int min, int max, int clip)
{
	int z;

	for (z=min; z<max; z+=clip)
	{
		// Mmmm.. cpu
		ShadeString(x,y,string,z);
		vid_waitvbl();
	}

	ShadeString(x,y,string,max);
}

void PrepArea(int n)
{
	int y;

	for (y=(n+20); y<240; y++)
	{
		DrawLine(y,GetColor(0,0,255));
	}
}

void Popup(int top, int bottom)
{
	int w, y, n;

	for (n=bottom; n>top; n-=2)
	{
		w = 0;

		for (y=n; y<(n+19); y++)
		{
			w++;
			DrawLine(y,GetColor(0,0,(w*13)));
		}
		
		vid_waitvbl();
	}

	PrepArea(top);
}

void PrettyBlueLineyBackground()
{
	int z;

	for (z=0; z<240; z+=2)
	{
		DrawLine(z,GetColor(0,0,z));
		DrawLine(z+1,GetColor(0,0,z/4));
	}
}

void WrapMsgs()
{
	int pos, lastsp, col, z;

	for (z=0; z<MESSAGES; z++)
	{
		col = 0;
		lastsp = 0;
		for (pos=0; pos<(strlen(messages[z]) - 1); pos++)
		{
			if (messages[z][pos] == ' ') lastsp = pos;

			col++;
			if (col > 35 && lastsp > 0)
			{
				messages[z][lastsp] = '\n';
				col = 35 - lastsp;
				lastsp = 0;
			}
		}
	}
}

void blorp()
{
	int z;

	for (z=0; z<STUFF; z++)
	{
		DrawChar((objs[z].x+2)*8,(objs[z].y+2)*8,objs[z].chr,objs[z].color);
	}
}

void ProductPlacement()
{
	int y,z,TheKitten;

	for (z=0; z<STUFF; z++)
	{
		objs[z].x = randnum(35);
		objs[z].y = randnum(16);
		objs[z].chr = randnum(67)+1;	// no invisible kittens
		objs[z].color = GetColor(randnum(192)+63,randnum(192)+63,randnum(192)+63);
		objs[z].Msg = randnum(MESSAGES);
		objs[z].IsKitten = 0;

		for (y=0; y<z; y++)
		{
			if ((objs[y].x == objs[z].x && objs[y].y == objs[z].y) || (objs[y].Msg == objs[z].Msg) || (objs[y].chr == objs[z].chr))
				z = -1;	// nuts.
		}
	}

	TheKitten = randnum(STUFF);
	objs[TheKitten].IsKitten = 1;

	blorp();
}

int collide(int x, int y)
{
	int z;

	for (z=0; z<STUFF; z++)
	{
		if (objs[z].x == x && objs[z].y == y)
			return z;
	}

	return -1;
}

int dc_main()
{
	int y, z, rx, ry, cx, cy, rc=128, ri=5, LastMsgShown;

//	dc_setup_quiet(DM_640x480, PM_RGB565);
	dc_setup_quiet(DM_320x240, PM_RGB565);
	serial_disable();
	mcont = maple_controller_addr();

	WrapMsgs();
	vid_clear(0,0,0);
	PrettyBlueLineyBackground();
	ShadeString(15,15,INSTRUCTIONS,255);

	while (1)
	{
		cont_get_cond(mcont, &cond);

		if (!(cond.buttons & CONT_START))
			break;

		seed++;
		vid_waitvbl();
	}

togo:	vid_clear(0,0,0);
	LastMsgShown = -1;
	Popup(150,240);
	ProductPlacement();

	while (1)
	{
		ry = randnum(16);
		rx = randnum(35);

		if (collide(rx,ry) == -1)
		{
			cx = rx;
			cy = ry;
			break;
		}
	}

	sleep(500);

	while (1)
	{
		cont_get_cond(mcont, &cond);

		if (!(cond.buttons & CONT_DPAD_UP))
			ry-=1;

		if (!(cond.buttons & CONT_DPAD_DOWN))
			ry+=1;

		if (!(cond.buttons & CONT_DPAD_LEFT))
			rx-=1;

		if (!(cond.buttons & CONT_DPAD_RIGHT))
			rx+=1;

		if (!(cond.buttons & CONT_START))
			break;

		if (rx != cx || ry != cy)
		{
			if (rx < 0) rx = 0;
			if (ry < 0) ry = 0;
			if (rx > 35) rx = 35;
			if (ry > 16) ry = 16;
			seed += (rx + ry);
			sleep(100);
		}

		y = collide(rx,ry);

		if (y < 0)	// if no collision, commit
		{
			DrawString((cx+2)*8,(cy+2)*8,"#",GetColor(0,0,0));
			cx=rx;
			cy=ry;
		}
		else		// otherwise, retract
		{
			if (objs[y].IsKitten == 1)	// fear not, kitten
			{
				for(z=20; z<154; z++)
				{
					PrepArea(150);
					DrawChar(z,182,3,GetColor(255,255,255));
					DrawChar((320-z),182,objs[y].chr,objs[y].color);
					vid_waitvbl();
					sleep(10);
				}

				FadeString(18,180,KITTEN,0,255,5);
				sleep(2500);
				seed++;
				goto togo;
//				return 0;
			}
			else if (y != LastMsgShown)
			{
				PrepArea(150);
				FadeString(18,180,messages[objs[y].Msg],0,255,10);
				LastMsgShown = y;
				seed += LastMsgShown;
			}

			rx=cx;
			ry=cy;
		}

		rc += ri;
		if (rc > 250 || rc < 63) ri = -ri;
		DrawChar((cx+2)*8,(cy+2)*8,3,GetColor(rc,rc,rc));
		vid_waitvbl();
	}

	return 0;
}
